"""
This module deals with indexing EASE-DGGS coordinates.

© Regents of the University of Minnesota. All rights reserved.
This software is released under an Apache 2.0 license. Further details about the Apache 2.0 license are available in the license.txt file.
"""
# import re


import numpy as np

# from geopandas import GeoSeries, GeoDataFrame
import geopandas as gpd
from shapely import wkt
from shapely.geometry import Point  # , Polygon

from vgrid.utils.easedggs.constants import (
    grid_spec,
    levels_specs,
    ease_crs,
    geo_crs,
    cell_scale_factors,
)

from vgrid.utils.easedggs.dggs.utils import (
    format_response,
    gen_point_grid,
    get_polygon_corners,
)

from vgrid.utils.easedggs.dggs.checks import (
    check_level,
    validate_coords_lon_lat,
    validate_grid_ids,
    check_coords_range,
)
from vgrid.utils.easedggs.dggs.transforms import (
    coords_lon_lat_to_coords_ease,
    coords_ease_to_coords_grid,
    grid_xy_coord_to_ease_coord,
)
from vgrid.utils.easedggs.logConfig import logger

######
#
# Main indexing functions
#
######


def geos_to_grid_ids(
    coords_lon_lat,
    level=0,
    levels_specs=levels_specs,
    source_crs=4326,
    target_crs=ease_crs,
):
    """
    Return the EASE-DGGS ID for the cell correpsonding with lon, lat pair.

    Parameters
    ----------
    coords_lon_lat : list
       List of longitude,latitude cooridnate pairs (lon, lat).
    level _ int
        The specific resolution in the heiararch for the corresponding grid cell. Default is 0, the coarsest resolution.
    levels_specs : dictonary
        The dictionary with paratmer and config options. Default is 'levels_specs'

    Returns
    -------
    grid_id : dict
        The grid IDs of the cells for the cooridnates, at the specified resolution that correspond with (lon, lat)
    """

    if not check_level(level):
        success = False
        data = ["The specified level is invalid."]

        return format_response(data, success)

    success, data = validate_coords_lon_lat(coords_lon_lat)
    if not success:
        return format_response(data, success)

    coords_ease = coords_lon_lat_to_coords_ease(coords_lon_lat)

    coords_grid = coords_ease_to_coords_grid(coords_ease)

    grid_ids = coords_grid.apply(lambda coord: _grid_xy_to_grid_id(coord, level=level))

    return format_response(grid_ids.tolist(), success)


def _gid_to_coord_ease(gid, cell_scale_factors=cell_scale_factors, centroid_offset=0.5):
    """
    Convert EASE-DGGS cell ID to ease_x, ease_y cooridnates.

    Parameters
    ----------
    grid_id : String
       EASE-DGGS cell ID.

    cell_scale_factors : numpy array
       Array with the level scaling factors .

    Returns
    -------
    ease_x, ease_y : tuple
        ease_x, ease_y cooridnates of the specified cell.
    """
    if not isinstance(gid, str):
        return False
    # centroid are half of the grid cell x,y
    # centroid_offset = 0.5

    id_elements = gid.split(".")

    # level is used in indexing the  pixel_scale_factors array
    #   add one to get the indexing right
    index_max = int(id_elements[0][1]) + 1

    id_elements.pop(0)

    # Level 0 has 3 digits for rows, columns. all other levels have 1 each
    #   deal with Level 0, then remove it from elements. then append the list
    y_row_elements = [float(id_elements[0][0:3])]
    x_col_elements = [float(id_elements[0][3:])]

    id_elements.pop(0)

    for id in id_elements:
        y_row_elements.append(float(id[0]))
        x_col_elements.append(float(id[1]))

    y_row_elements = np.array(y_row_elements)
    x_col_elements = np.array(x_col_elements)

    # remove unused levels from the arrays; makes book keeing easier
    cell_scale_factors = cell_scale_factors[0:index_max]

    y_row_scaled = y_row_elements * cell_scale_factors
    x_col_scaled = x_col_elements * cell_scale_factors

    grid_offset = cell_scale_factors[-1] * centroid_offset

    x_grid = np.cumsum(x_col_scaled)[-1] + grid_offset
    y_grid = np.cumsum(y_row_scaled)[-1] + grid_offset
    #     print('x_grid: {}; y_grid: {}'.format(x_grid, y_grid))
    logger.debug("x_grid: {}; y_grid: {}".format(x_grid, y_grid))

    ease_x, ease_y = grid_xy_coord_to_ease_coord(x_grid=x_grid, y_grid=y_grid)

    return Point(ease_x, ease_y)  # , grid_offset


def grid_ids_to_ease(
    grid_ids, cell_scale_factors=cell_scale_factors, target_crs=ease_crs
):
    """
    Convert a list of EASE-DGGS IDs to EASE Grid v2 cooridnates: Point(ease_x, ease_y).

    Parameters
    ----------
    grid_ids : List
       List of EASE-DGGS cell IDs to convert to Point(ease_x, ease_y).

    cell_scale_factors : numpy array
       Array with the level scaling factors for each level.

    Returns
    -------
    coords_ease : GeoSeries
        GeoSeies of coordinates (ease_x, ease_y) for corresponding grid IDs.
    """

    if not isinstance(grid_ids, list):
        return False

    coords_ease = [_gid_to_coord_ease(gid) for gid in grid_ids]

    coords_ease = gpd.GeoSeries(coords_ease, crs=target_crs)

    return coords_ease


def grid_ids_to_geos(
    grid_ids,
    cell_scale_factors=cell_scale_factors,
    source_crs=ease_crs,
    target_crs=geo_crs,
):
    """
    Convert EASE-DGGS cell ID to ease_x, ease_y cooridnates.

    Parameters
    ----------
    grid_ids : List
       List of EASE-DGGS cell IDs to convert to (lon, lat).

    cell_scale_factors : numpy array
       Array with the level scaling factors for each level.

    Returns
    -------
    coords_lon_lat : dict
        Geographic coordinates (lon, lat) corresponding withe grid ID.
    """

    success, data = validate_grid_ids(grid_ids)
    if not success:
        return format_response(data, success)

    coords_ease = grid_ids_to_ease(grid_ids)

    coords_lon_lat = coords_ease.to_crs(target_crs)
    coords_lon_lat = coords_lon_lat.apply(lambda coord: (coord.x, coord.y))
    data = coords_lon_lat.to_list()

    return format_response(data, success)


def _grid_xy_to_grid_id(grid_xy, level=0):
    """
    Convert a EASE-DGGS coordinate (x, y) into corresponding Grid ID for specified level;

    Parameters
    ----------
    grid_xy : Shapley point(x,y)
        EASE-DGGS cooridnate point

    level : str, optional
        EASE-DGGS level/resolution. Defaults to 0.

    Returns
    ----------
    grid_id : str
        The EASE-DGGS id corrsponding to the supplied EASE-DGGS coordinate (x, y).
    """
    if not isinstance(grid_xy, Point):
        return False

    # small number problem. This is section comes from a discussion
    #   with David Porter & other GEMS devs. Basically, it is  way
    #   of arriving at a calculated way of determing where to round
    #
    # the calculation below effectively determins that 8 decimal places
    #   is a reasonable place to round. The assumption is the EASE grid
    #   values will be anywhere between min/max, which is the space for values
    #   with the precision we're working with from pyproj etc, the 10e15 term
    #   is used to get a sense of what a small number in EASE v2 is
    #
    # in pract, rounding to 8 decimals in the grid_xy space (basically, index space)
    #   means we can detect differences in coordinates on the order of milimeters.
    #   so far, simmple testing has meant that this should be good enough enough for
    #   determining multiples in grid_xy space using np.divmod
    #
    r_digit = 6  # 6 works; 8 causes problems with y dims
    x = np.around(grid_xy.x, decimals=r_digit)
    y = np.around(grid_xy.y, decimals=r_digit)
    np.set_printoptions(precision=6)

    # small rounding differences around 0 were causing problems.
    #   in particualr, along the left hand edge of the grid,
    #   some of the x values were negative, and slightly less
    #   than 0. because np.divmod (used for dividing the cell)
    #   uses a floor operation, those numbers that were < 0, the floor
    #   results in them being -1,not 0. This can be corrected with rounding
    #   or other solutions, but since the grid is only defined for positive x,y
    #   values, the simplist solution is for
    if x < 0.0:
        x = 0.0

    if y < 0.0:
        y = 0.0

    grid_cell_id = ["L{}".format(level)]

    # for every level between 0 and supplied level, inclusive
    #   want to figure our the row/column index. see documentation
    #   on the actual scheme
    for lv in range(0, level + 1, 1):
        #         lv = lv
        lv = int(lv)

        # because coordinate were transformed above, nothing needed here.
        #   both x,y integer-parts represent the major Level 0 grid cell
        #       in grid cooridnate space
        #
        #   the float-part indicate the sub-location in that pixel. later, the
        #      the mod will be multipled by the refine_ratio, which will give the
        #      location in the finer mesh grids
        x_div, x_mod = np.divmod(x, 1)
        y_div, y_mod = np.divmod(y, 1)

        if lv == 0:
            # the first Level has a 3 digit space for row, columns
            #    RRRCCC format
            rc = "{:03d}{:03d}".format(int(y_div), int(x_div))

        else:
            # everything Level after L0 uses a single integer for row, column index
            #   RC format
            rc = "{:d}{:d}".format(int(y_div), int(x_div))

        grid_cell_id.append("{}".format(str(rc)))

        # each time after first, we need to multiple the mod by refine_ratio value
        x = np.around(x_mod * levels_specs[lv]["refine_ratio"], decimals=r_digit)
        y = np.around(y_mod * levels_specs[lv]["refine_ratio"], decimals=r_digit)

    seperator = "."

    return seperator.join(grid_cell_id)


def ease_polygon_to_grid_ids(
    polygon_ease, level=0, source_crs=ease_crs, levels_specs=levels_specs, wkt_geom=True
):
    """
    Identify all grid cell IDs that correspond with supplied polygon

    Parameters
    ----------
    polygon : WKT
       WKT the polygon to convert to grid cell IDs.

    level _ int
        The grid level of constituent cell IDs to return

    Returns
    -------
    Grid IDs: dict
        Grid cell IDs for all constituent cells at the specified level.
    """
    if not check_level(level):
        success = False
        data = ["The specified level is invalid."]

        return success, data

    if not wkt_geom:
        success = False
        data = ["Expected the polygon in Well Known Text (WKT) format."]

        return format_response(data, success)
    # convert to EASE Grid
    if wkt_geom:
        polygon_ease = wkt.loads(polygon_ease)

    if not (
        (polygon_ease.geom_type == "MultiPolygon")
        or (polygon_ease.geom_type == "Polygon")
    ):
        success = False
        data = ["The input geometry should be a Polygon or MultiPolygon."]

        return format_response(data, success)

    if source_crs != 6933:
        success = False
        data = ["The expected source crs is 6933."]

        return format_response(data, success)

    success = True

    # having issues where corner coords are not quite behaving
    #   as might be expected. cell L0.202482 is causing difficulties
    #   when trying to map the locations of conrers, I am going to add
    #   small buffer to the polygon, to try and fix the problem
    #   will add 1/2 the smallest grid cell size. cap_type = 3 is
    #   square buffer
    fin_lev = list(levels_specs)[-1]
    buff_dist = levels_specs[fin_lev]["x_length"] * 0.5
    polygon_ease_in = polygon_ease
    polygon_ease = polygon_ease.buffer(buff_dist, cap_style=3)

    # poly = poly.to_crs(target_crs)

    # get corner coordiantes in EASE
    corner_coords_ease = get_polygon_corners(polygon_ease)

    corner_coords_ease = gpd.GeoSeries(corner_coords_ease, crs=source_crs)

    corner_coords_grid = coords_ease_to_coords_grid(corner_coords_ease)
    corner_grid_ids = corner_coords_grid.apply(
        lambda coord: _grid_xy_to_grid_id(coord, level=level)
    )

    corner_centroids = grid_ids_to_ease(corner_grid_ids.to_list())

    # get x, y deltas for grid points; these are the distances between ceintroids
    delta_x = levels_specs[level]["x_length"]
    delta_y = levels_specs[level]["y_length"]

    point_grid = gen_point_grid(
        upper_left=corner_centroids[0],
        upper_right=corner_centroids[1],
        lower_right=corner_centroids[2],
        lower_left=corner_centroids[3],
        delta_x=delta_x,
        delta_y=delta_y,
    )

    # ok, below presumes dtaframes, but the shapes input is not
    # RETHING df use, and just use the shapely options?
    polygon_ease_in = gpd.GeoDataFrame(
        {"name": ["test_shape"], "geometry": polygon_ease_in}, crs=source_crs
    )

    # points_in_poly = gpd.sjoin(point_grid, polygon_ease_in, how='left', op='within')
    points_in_poly = gpd.sjoin(
        point_grid, polygon_ease_in, how="left", predicate="within"
    )
    points_in_poly = points_in_poly.dropna().reset_index()

    # once the sjoin is done, there is not a lot of point carting around the df framework.
    #   may not make much difference on the ultimate performance, but no need to cart it around
    points_in_poly = points_in_poly.geometry

    coords_grid = coords_ease_to_coords_grid(points_in_poly)
    grid_ids = coords_grid.apply(lambda coord: _grid_xy_to_grid_id(coord, level=level))

    return format_response(grid_ids.to_list(), success)


def geo_polygon_to_grid_ids(
    polygon_lon_lat,
    level=0,
    source_crs=geo_crs,
    target_crs=ease_crs,
    levels_specs=levels_specs,
    return_centroids=True,
    wkt_geom=True,
):
    """
    Identify all grid cell IDs that correspond with the supplied polygon (lon, lat).

    Parameters
    ----------
    polygon : WKT
       WKT the polygon to convert to grid cell IDs.

    level _ int
        The grid level of constituent cell IDs to return

    source_crs : int
        The source EPSG code for the polygon. Default is 4326 (lon, lat).

    target_crs : int
        The targest EPSG code for the polygon. Default is 6933 (EASE Grid v2)

    Returns
    -------
    Grid IDs : dict
        Grid cell IDs for all constituent cells at the specified level.
    """
    # convert to EASE Grid
    if not wkt_geom:
        success = False
        data = ["Expected the polygon in Well Known Text (WKT) format."]

        return format_response(data, success)

    if wkt_geom:
        polygon_lon_lat = wkt.loads(polygon_lon_lat)

    if source_crs != 4326:
        success = False
        data = ["The expected source crs is 4326."]

        return format_response(data, success)

    corners_lon_lat = get_polygon_corners(polygon_lon_lat)
    corners_lon_lat = [(coord.x, coord.y) for coord in corners_lon_lat]

    if not check_coords_range(corners_lon_lat):
        success = False

        data = f"""Lon range is {grid_spec["geo"]["min_x"]} : {grid_spec["geo"]["max_x"]} ; lat range is {grid_spec["geo"]["max_y"]} : {grid_spec["geo"]["min_y"]}"""
        data = [data]

    polygon_lon_lat = gpd.GeoSeries(polygon_lon_lat, crs=source_crs)

    polygon_ease = polygon_lon_lat.to_crs(target_crs)
    polygon_ease = polygon_ease.geometry.values[0].wkt

    response = ease_polygon_to_grid_ids(polygon_ease, level=level)

    return response



# -*- coding: utf-8 -*-

"""
This Python 3.3 module analyzes angular, linear, and areal distortion of various map projections.
CHANGELOG:
- Alexander Raichev (AR), 2013-02-04: Initial version.
- AR, 2013-07-23: Ported to Python 3.3.
"""
# *****************************************************************************
#       Copyright (C) 2013 Alexander Raichev <alex.raichev@gmail.com>
#
#  Distributed under the terms of the GNU Lesser General Public License (LGPL)
#                  http://www.gnu.org/licenses/
# *****************************************************************************

# Import my modules.
from .ellipsoids import EMR_SPHERE, WGS84_ELLIPSOID
from .projection_wrapper import Proj
from .distortion import distortion_stats

if __name__ == "__main__":
    # List of map projections to analyze.
    # They must be implemented in the pyproj third-party Python module or
    # homemade, e.g. csea as implemented in pj_csea.py.
    projections = [
        ("HEALPix", "healpix"),
        ("rHEALPix", "rhealpix"),
        ("Lambert cylindrical equal area", "cea"),
        ("Lambert azimuthal equal area", "laea"),
        ("Quadrilateralized spherical cube", "qsc"),
        ("Cubic Snyder equal area", "csea"),
        ("Icosahedral Snyder equal area", "isea"),
        ("Mercator", "merc"),
        ("Stereographic", "stere"),
        ("Lambert conformal conic", "lcc"),
        ("Equirectangular", "eqc"),
        ("Azimuthal equidistant", "aeqd"),
        ("Winkel triple", "wintri"),
    ]

    N = 30000
    for E in [EMR_SPHERE, WGS84_ELLIPSOID]:
        print(E)
        sample = [E.random_point(phi_min=-89.5, phi_max=89.5) for i in range(N)]
        for name, proj in projections:
            f = Proj(E, proj)
            stats = distortion_stats(f, sample)[1]
            print((name + " (" + proj + "):"))
            print(stats)



import json
import argparse
from vgrid.utils import olc
from tqdm import tqdm
from shapely.geometry import shape, box, Polygon
from vgrid.generator.settings import max_cells, graticule_dggs_to_feature
from shapely.ops import unary_union


def calculate_total_cells(resolution, bbox):
    """Calculate the total number of cells within the bounding box for a given resolution."""
    area = olc.decode(
        olc.encode(bbox[1], bbox[0], resolution)
    )  # Use bbox min lat, min lon for the area
    lat_step = area.latitudeHi - area.latitudeLo
    lng_step = area.longitudeHi - area.longitudeLo

    sw_lng, sw_lat, ne_lng, ne_lat = bbox
    total_lat_steps = int((ne_lat - sw_lat) / lat_step)
    total_lng_steps = int((ne_lng - sw_lng) / lng_step)

    return total_lat_steps * total_lng_steps


def generate_grid(resolution, verbose=True):
    """
    Generate a global grid of Open Location Codes (Plus Codes) at the specified precision
    as a GeoJSON-like feature collection.
    """
    # Define the boundaries of the world
    sw_lat, sw_lng = -90, -180
    ne_lat, ne_lng = 90, 180

    # Get the precision step size
    area = olc.decode(olc.encode(sw_lat, sw_lng, resolution))
    lat_step = area.latitudeHi - area.latitudeLo
    lng_step = area.longitudeHi - area.longitudeLo

    olc_features = []

    # Calculate the total number of steps for progress tracking
    total_lat_steps = int((ne_lat - sw_lat) / lat_step)
    total_lng_steps = int((ne_lng - sw_lng) / lng_step)
    total_steps = total_lat_steps * total_lng_steps

    with tqdm(
        total=total_steps,
        desc="Generating OLC DGGS",
        unit=" cells",
        disable=not verbose,
    ) as pbar:
        lat = sw_lat
        while lat < ne_lat:
            lng = sw_lng
            while lng < ne_lng:
                # Generate the Plus Code for the center of the cell
                center_lat = lat + lat_step / 2
                center_lon = lng + lng_step / 2
                olc_id = olc.encode(center_lat, center_lon, resolution)
                resolution = olc.decode(olc_id).codeLength
                cell_polygon = Polygon(
                    [
                        [lng, lat],  # SW
                        [lng, lat + lat_step],  # NW
                        [lng + lng_step, lat + lat_step],  # NE
                        [lng + lng_step, lat],  # SE
                        [lng, lat],  # Close the polygon
                    ]
                )
                olc_feature = graticule_dggs_to_feature(
                    "olc", olc_id, resolution, cell_polygon
                )
                olc_features.append(olc_feature)
                lng += lng_step
                pbar.update(1)  # Update progress bar
            lat += lat_step

    # Return the feature collection
    return {"type": "FeatureCollection", "features": olc_features}


def generate_grid_within_bbox(resolution, bbox):
    """
    Generate a grid of Open Location Codes (Plus Codes) within the specified bounding box.
    """
    min_lon, min_lat, max_lon, max_lat = bbox
    bbox_poly = box(min_lon, min_lat, max_lon, max_lat)

    # Step 1: Generate base cells at the lowest resolution (e.g., resolution 2)
    base_resolution = 2
    base_cells = generate_grid(base_resolution, verbose=False)

    # Step 2: Identify seed cells that intersect with the bounding box
    seed_cells = []
    for base_cell in base_cells["features"]:
        base_cell_poly = Polygon(base_cell["geometry"]["coordinates"][0])
        if bbox_poly.intersects(base_cell_poly):
            seed_cells.append(base_cell)

    refined_features = []

    # Step 3: Iterate over seed cells and refine to the output resolution
    for seed_cell in seed_cells:
        seed_cell_poly = Polygon(seed_cell["geometry"]["coordinates"][0])

        if seed_cell_poly.contains(bbox_poly) and resolution == base_resolution:
            # Append the seed cell directly if fully contained and resolution matches
            refined_features.append(seed_cell)
        else:
            # Refine the seed cell to the output resolution and add it to the output
            refined_features.extend(
                refine_cell(
                    seed_cell_poly.bounds, base_resolution, resolution, bbox_poly
                )
            )

    resolution_features = [
        feature
        for feature in refined_features
        if feature["properties"]["resolution"] == resolution
    ]

    final_features = []
    seen_olc_ids = set()  # Reset the set for final feature filtering

    for feature in resolution_features:
        olc_id = feature["properties"]["olc"]
        if olc_id not in seen_olc_ids:  # Check if OLC code is already in the set
            final_features.append(feature)
            seen_olc_ids.add(olc_id)

    return {"type": "FeatureCollection", "features": final_features}


def refine_cell(bounds, current_resolution, target_resolution, bbox_poly):
    """
    Refine a cell defined by bounds to the target resolution, recursively refining intersecting cells.
    """
    min_lon, min_lat, max_lon, max_lat = bounds
    if current_resolution < 10:
        valid_resolution = current_resolution + 2
    else:
        valid_resolution = current_resolution + 1

    area = olc.decode(olc.encode(min_lat, min_lon, valid_resolution))
    lat_step = area.latitudeHi - area.latitudeLo
    lng_step = area.longitudeHi - area.longitudeLo

    olc_features = []
    lat = min_lat
    while lat < max_lat:
        lng = min_lon
        while lng < max_lon:
            # Define the bounds of the finer cell
            finer_cell_bounds = (lng, lat, lng + lng_step, lat + lat_step)
            finer_cell_poly = box(*finer_cell_bounds)

            if bbox_poly.intersects(finer_cell_poly):
                # Generate the Plus Code for the center of the finer cell
                center_lat = lat + lat_step / 2
                center_lon = lng + lng_step / 2
                olc_id = olc.encode(center_lat, center_lon, valid_resolution)
                resolution = olc.decode(olc_id).codeLength

                cell_polygon = Polygon(
                    [
                        [lng, lat],  # SW
                        [lng, lat + lat_step],  # NW
                        [lng + lng_step, lat + lat_step],  # NE
                        [lng + lng_step, lat],  # SE
                        [lng, lat],  # Close the polygon
                    ]
                )

                olc_feature = graticule_dggs_to_feature(
                    "olc", olc_id, resolution, cell_polygon
                )
                olc_features.append(olc_feature)

                # Recursively refine the cell if not at target resolution
                if valid_resolution < target_resolution:
                    olc_features.extend(
                        refine_cell(
                            finer_cell_bounds,
                            valid_resolution,
                            target_resolution,
                            bbox_poly,
                        )
                    )

            lng += lng_step
            # pbar.update(1)
        lat += lat_step

    return olc_features


def generate_grid_resample(resolution, geojson_features):
    """
    Generate a grid of Open Location Codes (Plus Codes) within the specified GeoJSON features.
    """
    # Step 1: Union all input geometries
    geometries = [
        shape(feature["geometry"]) for feature in geojson_features["features"]
    ]
    unified_geom = unary_union(geometries)

    # Step 2: Generate base cells at the lowest resolution (e.g., resolution 2)
    base_resolution = 2
    base_cells = generate_grid(base_resolution, verbose=True)

    # Step 3: Identify seed cells that intersect with the unified geometry
    seed_cells = []
    for base_cell in base_cells["features"]:
        base_cell_poly = Polygon(base_cell["geometry"]["coordinates"][0])
        if unified_geom.intersects(base_cell_poly):
            seed_cells.append(base_cell)

    refined_features = []

    # Step 4: Refine seed cells to the desired resolution
    for seed_cell in seed_cells:
        seed_cell_poly = Polygon(seed_cell["geometry"]["coordinates"][0])

        if seed_cell_poly.contains(unified_geom) and resolution == base_resolution:
            refined_features.append(seed_cell)
        else:
            refined_features.extend(
                refine_cell(
                    seed_cell_poly.bounds, base_resolution, resolution, unified_geom
                )
            )

    # Step 5: Filter features to keep only those at the desired resolution and remove duplicates
    resolution_features = [
        feature
        for feature in refined_features
        if feature["properties"]["resolution"] == resolution
    ]

    final_features = []
    seen_olc_ids = set()

    for feature in resolution_features:
        olc_id = feature["properties"]["olc"]
        if olc_id not in seen_olc_ids:
            final_features.append(feature)
            seen_olc_ids.add(olc_id)

    return {"type": "FeatureCollection", "features": final_features}


def main():
    parser = argparse.ArgumentParser(
        description="Generate OpenLocationCode/ Google Pluscode DGGS."
    )
    parser.add_argument(
        "-r",
        "--resolution",
        type=int,
        choices=[2, 4, 6, 8, 10, 11, 12, 13, 14, 15],
        default=8,
        help="Resolution [2, 4, 6, 8, 10, 11, 12, 13, 14, 15]",
    )

    parser.add_argument(
        "-b",
        "--bbox",
        type=float,
        nargs=4,
        help="Bounding box in the format: min_lon min_lat max_lon max_lat (default is the whole world)",
    )

    args = parser.parse_args()
    resolution = args.resolution
    bbox = args.bbox if args.bbox else [-180, -90, 180, 90]

    num_cells = calculate_total_cells(resolution, bbox)

    if bbox == [-180, -90, 180, 90]:
        print(f"Resolution {resolution} will generate {num_cells} cells ")

        if num_cells > max_cells:
            print(f"which exceeds the limit of {max_cells}. ")
            print("Please select a smaller resolution and try again.")
            return
        geojson_features = generate_grid(resolution, verbose=True)

    else:
        geojson_features = generate_grid_within_bbox(resolution, bbox)

    geojson_path = f"olc_grid_{resolution}.geojson"
    with open(geojson_path, "w") as f:
        json.dump(geojson_features, f, indent=2)

    print(f"OLC grid saved to {geojson_path}")


if __name__ == "__main__":
    main()



import locale
import argparse
import csv
from shapely.wkt import loads
from vgrid.conversion.latlon2dggs import latlon2isea3h
from texttable import Texttable
import platform
from pyproj import Geod
import math
geod = Geod(ellps="WGS84")

locale.setlocale(locale.LC_ALL, "")

if platform.system() == "Windows":
    from vgrid.utils.eaggr.enums.shape_string_format import ShapeStringFormat
    from vgrid.utils.eaggr.eaggr import Eaggr
    from vgrid.utils.eaggr.shapes.dggs_cell import DggsCell
    from vgrid.utils.eaggr.enums.model import Model


def fix_eaggr_wkt(eaggr_wkt):
    # Extract the coordinate section
    coords_section = eaggr_wkt[eaggr_wkt.index("((") + 2 : eaggr_wkt.index("))")]
    coords = coords_section.split(",")
    # Append the first point to the end if not already closed
    if coords[0] != coords[-1]:
        coords.append(coords[0])
    fixed_coords = ", ".join(coords)
    return f"POLYGON (({fixed_coords}))"


def isea3h_metrics(isea3h_dggs, res):
    num_cells = 20 * (7**res)
    lat, lon = 10.775275567242561, 106.70679737574993

    isea3h_cell = DggsCell(latlon2isea3h(lat, lon, res))

    cell_to_shp = isea3h_dggs.convert_dggs_cell_outline_to_shape_string(
        isea3h_cell, ShapeStringFormat.WKT
    )
    cell_to_shp_fixed = fix_eaggr_wkt(cell_to_shp)
    cell_polygon = loads(cell_to_shp_fixed)

    isea3h2point = isea3h_dggs.convert_dggs_cell_to_point(isea3h_cell)
    accuracy = isea3h2point.get_accuracy()

    avg_area = abs(
        geod.geometry_area_perimeter(cell_polygon)[0]
    )  # Area in square meters
    avg_edge_length = (
        abs(geod.geometry_area_perimeter(cell_polygon)[1]) / 6
    )  # Perimeter in meters/ 6
    if res == 0:
        avg_edge_length = (
            abs(geod.geometry_area_perimeter(cell_polygon)[1]) / 3
        )  # icosahedron faces

    return num_cells, avg_edge_length, avg_area
    # earth_surface_area_km2 = 510_065_621.724  # 510.1 million square kilometers
    # num_cells = 20 * (7**res)
    # avg_area = (earth_surface_area_km2 / num_cells) * (10**6)
    # avg_edge_length = math.sqrt((2 * avg_area) / (3 * math.sqrt(3)))
    # return num_cells, avg_edge_length, avg_area



def isea3h_stats(isea3h_dggs, output_file=None):
    min_res = 0
    max_res = 40
    t = Texttable()
    # Add header to the table, including the new 'Cell Width' and 'Cell Area' columns
    t.add_row(
        [
            "Resolution",
            "Number of Cells",
            "Avg Edge Length (m)",
            "Avg Cell Area (sq m)",
            # "Accucracy",
        ]
    )

    # Check if an output file is specified (for CSV export)
    if output_file:
        # Open the output CSV file for writing
        with open(output_file, mode="w", newline="") as file:
            writer = csv.writer(file)
            writer.writerow(
                [
                    "Resolution",
                    "Number of Cells",
                    "Avg Edge Length (m)",
                    "Avg Cell Area (sq m)",
                    # "Accucracy",
                ]
            )

            for res in range(min_res, max_res + 1):
                num_cells, avg_edge_length, avg_area = isea3h_metrics(
                    isea3h_dggs, res
                )
                writer.writerow([res, num_cells, avg_edge_length, avg_area, accuracy])
        print(f"OpenEAGGGR ISEA3H stats saved to {output_file}.")
    else:
        for res in range(min_res, max_res + 1):
            num_cells, avg_edge_length, avg_area = isea3h_metrics(
                isea3h_dggs, res
            )
            formatted_num_cells = locale.format_string("%d", num_cells, grouping=True)
            formatted_edge_length = locale.format_string(
                "%.3f", avg_edge_length, grouping=True
            )
            formatted_area = locale.format_string("%.3f", avg_area, grouping=True)
            # formatted_accuracy = locale.format_string("%.3f", accuracy, grouping=True)
            # Add a row to the table
            t.add_row(
                [
                    res,
                    formatted_num_cells,
                    formatted_edge_length,
                    formatted_area,
                    #formatted_accuracy,
                ]
            )

        # Print the formatted table to the console
        print(t.draw())


def main():
    # Set up command-line argument parsing
    parser = argparse.ArgumentParser(
        description="Export or display OpenEAGGR ISEA3H DGGS stats."
    )
    parser.add_argument("-o", "--output", help="Output CSV file name.")
    args = parser.parse_args()
    if platform.system() == "Windows":
        isea3h_dggs = Eaggr(Model.ISEA3H)
        # Call the function with the provided output file (if any)
        isea3h_stats(isea3h_dggs, args.output)


if __name__ == "__main__":
    main()



import os
import argparse
import json
from tqdm import tqdm
import rasterio
from vgrid.utils import tilecode, mercantile
import numpy as np
from shapely.geometry import Polygon
from vgrid.stats.tilecodestats import tilecode_metrics
from vgrid.generator.settings import graticule_dggs_to_feature
from math import cos, radians
import csv
import re


def get_nearest_tilecode_resolution(raster_path):
    with rasterio.open(raster_path) as src:
        transform = src.transform
        crs = src.crs
        pixel_width = transform.a
        pixel_height = -transform.e
        cell_size = pixel_width * pixel_height

        if crs.is_geographic:
            # Latitude of the raster center
            center_latitude = (src.bounds.top + src.bounds.bottom) / 2
            # Convert degrees to meters
            meter_per_degree_lat = 111_320  # Roughly 1 degree latitude in meters
            meter_per_degree_lon = meter_per_degree_lat * cos(radians(center_latitude))

            pixel_width_m = pixel_width * meter_per_degree_lon
            pixel_height_m = pixel_height * meter_per_degree_lat
            cell_size = pixel_width_m * pixel_height_m

    nearest_resolution = None
    min_diff = float("inf")

    # Check resolutions from 0 to 26
    for res in range(27):
        _, _, avg_area = tilecode_metrics(res)
        diff = abs(avg_area - cell_size)
        # If the difference is smaller than the current minimum, update the nearest resolution
        if diff < min_diff:
            min_diff = diff
            nearest_resolution = res

    return nearest_resolution


def convert_numpy_types(obj):
    """Recursively convert NumPy types to native Python types"""
    if isinstance(obj, np.generic):
        return obj.item()  # Convert numpy types like np.uint8 to native Python int
    elif isinstance(obj, dict):
        return {k: convert_numpy_types(v) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [convert_numpy_types(v) for v in obj]
    else:
        return obj


def raster2tilecode(raster_path, resolution=None, format="geojson"):
    """Convert raster to tilecode format

    Args:
        raster_path (str): Path to input raster file
        resolution (int, optional): Tilecode resolution level [0-26]. If None, will be determined automatically
        format (str, optional): Output format, either 'geojson' or 'csv'. Defaults to 'geojson'

    Returns:
        dict: For geojson format, returns a GeoJSON FeatureCollection
        str: For csv format, returns a CSV string
    """
    # Step 1: Determine the nearest tilecode resolution if none is provided
    if resolution is None:
        resolution = get_nearest_tilecode_resolution(raster_path)
        print(f"Nearest tilecode resolution determined: {resolution}")

    # Open the raster file to get metadata and data
    with rasterio.open(raster_path) as src:
        raster_data = src.read()  # Read all bands
        transform = src.transform
        width, height = src.width, src.height
        band_count = src.count  # Number of bands in the raster

    tilecode_ids = set()

    for row in range(height):
        for col in range(width):
            lon, lat = transform * (col, row)
            tilecode_id = tilecode.latlon2tilecode(lat, lon, resolution)
            tilecode_ids.add(tilecode_id)

    # Sample the raster values at the centroids of the tilecode hexagons
    tilecode_data = []

    for tilecode_id in tqdm(tilecode_ids, desc="Resampling", unit=" cells"):
        # Get the centroid of the tilecode cell
        centroid_lat, centroid_lon = tilecode.tilecode2latlon(tilecode_id)

        # Sample the raster values at the centroid (lat, lon)
        col, row = ~transform * (centroid_lon, centroid_lat)

        if 0 <= col < width and 0 <= row < height:
            # Get the values for all bands at this centroid
            values = raster_data[:, int(row), int(col)]
            tilecode_data.append(
                {
                    "tilecode": tilecode_id,
                    **{
                        f"band_{i + 1}": values[i] for i in range(band_count)
                    },  # Create separate columns for each band
                }
            )

    if format == "csv":
        import io

        output = io.StringIO()
        if tilecode_data:
            writer = csv.DictWriter(output, fieldnames=tilecode_data[0].keys())
            writer.writeheader()
            writer.writerows(tilecode_data)
        return output.getvalue()

    # Create the GeoJSON-like structure
    tilecode_features = []
    for data in tqdm(tilecode_data, desc="Converting to GeoJSON", unit=" cells"):
        tilecode_id = data["tilecode"]
        match = re.match(r"z(\d+)x(\d+)y(\d+)", tilecode_id)
        if match:
            # Convert matched groups to integers
            z = int(match.group(1))
            x = int(match.group(2))
            y = int(match.group(3))

            # Get the bounds of the tile in (west, south, east, north)
            bounds = mercantile.bounds(x, y, z)
            if bounds:
                # Create the bounding box coordinates for the polygon
                min_lat, min_lon = bounds.south, bounds.west
                max_lat, max_lon = bounds.north, bounds.east
                cell_polygon = Polygon(
                    [
                        [min_lon, min_lat],  # Bottom-left corner
                        [max_lon, min_lat],  # Bottom-right corner
                        [max_lon, max_lat],  # Top-right corner
                        [min_lon, max_lat],  # Top-left corner
                        [
                            min_lon,
                            min_lat,
                        ],  # Closing the polygon (same as the first point)
                    ]
                )

                cell_resolution = z
                tilecode_feature = graticule_dggs_to_feature(
                    "tilecode", tilecode_id, cell_resolution, cell_polygon
                )
                band_properties = {
                    f"band_{i + 1}": data[f"band_{i + 1}"] for i in range(band_count)
                }
                tilecode_feature["properties"].update(
                    convert_numpy_types(band_properties)
                )
                tilecode_features.append(tilecode_feature)

    return {
        "type": "FeatureCollection",
        "features": tilecode_features,
    }


def raster2tilecode_cli():
    parser = argparse.ArgumentParser(
        description="Convert Raster in Geographic CRS to Tilecode DGGS"
    )
    parser.add_argument("-raster", type=str, required=True, help="Raster file path")

    parser.add_argument(
        "-r",
        "--resolution",
        type=int,
        required=False,
        default=None,
        help="Resolution [0..26]",
    )

    parser.add_argument(
        "-f",
        "--format",
        type=str,
        required=False,
        default="geojson",
        choices=["geojson", "csv"],
        help="Output format: 'geojson' for GeoJSON format with geometry, 'csv' for tabular data with tilecode and band values",
    )

    args = parser.parse_args()
    raster = args.raster
    resolution = args.resolution
    format = args.format.lower()

    if not os.path.exists(raster):
        print(f"Error: The file {raster} does not exist.")
        return

    if resolution is not None:
        if resolution < 0 or resolution > 26:
            print("Please select a resolution in [0..26] range and try again ")
            return

    result = raster2tilecode(raster, resolution, format)

    output_name = os.path.splitext(os.path.basename(raster))[0]
    output_path = f"{output_name}2tilecode.{format}"

    if format.lower() == "csv":
        with open(output_path, "w", newline="") as f:
            f.write(result)
    else:  # geojson
        with open(output_path, "w") as f:
            json.dump(result, f)

    print(f"Output saved as {output_path}")



"""
Copyright (c) Riskaware 2015

This file is part of OpenEAGGR.

OpenEAGGR is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

OpenEAGGR is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

A copy of the GNU Lesser General Public License is available in COPYING.LESSER
or can be found at <http://www.gnu.org/licenses/>.
"""

from vgrid.utils.eaggr.enums.dggs_shape_type import DggsShapeType
from vgrid.utils.eaggr.enums.dggs_shape_location import check_dggs_shape_location
from vgrid.utils.eaggr.shapes.dggs_cell import DggsCell
from vgrid.utils.eaggr.shapes.dggs_linestring import DggsLinestring
from vgrid.utils.eaggr.shapes.dggs_polygon import DggsPolygon


## Shape defined by DGGS cells.
#
#  The class wraps one of a DggsCell, DggsLinestring or DggsPolygon
#  (depending on the type of the lat/long shape that was coverted by the library).
#  The type of shape stored can be found from the get_shape_type() method.
class DggsShape(object):
    ## Sets the shape, along with its type and location.
    #  @param shape Must be DggsCell, DggsLinestring or DggsPolygon.
    #  @param location Must be one of the DggsShapeLocation values.
    #  @throw ValueError Thrown if input arguments are not one of the expected types.
    def __init__(self, shape, location):
        if isinstance(shape, DggsCell):
            self._shape_type = DggsShapeType.CELL
        elif isinstance(shape, DggsLinestring):
            self._shape_type = DggsShapeType.LINESTRING
        elif isinstance(shape, DggsPolygon):
            self._shape_type = DggsShapeType.POLYGON
        else:
            raise ValueError(
                "First input argument to DggsShape() must be DggsCell, DggsLinestring or DggsPolygon"
            )
        self._shape = shape
        check_dggs_shape_location(location)
        self._location = location

    ## @return One of the DggsShapeType values.
    def get_shape_type(self):
        return self._shape_type

    ## @return Object of either DggsCell, DggsLinestring or DggsPolygon.
    def get_shape(self):
        return self._shape

    ## Returns a value indicating where the shape sits in relation to the faces
    #  of the polyhedral globe.
    #  @return One of the DggsShapeLocation values.
    def get_location(self):
        return self._location



"""
Copyright (c) Riskaware 2015

This file is part of OpenEAGGR.

OpenEAGGR is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

OpenEAGGR is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

A copy of the GNU Lesser General Public License is available in COPYING.LESSER
or can be found at <http://www.gnu.org/licenses/>.
"""

import unittest
from vgrid.utils.eaggr.shapes.dggs_cell import DggsCell
from vgrid.utils.eaggr.shapes.dggs_linestring import DggsLinestring
from vgrid.utils.eaggr.shapes.dggs_polygon import DggsPolygon


## Unit tests for the DggsPolygon class.
class DggsPolygonTest(unittest.TestCase):
    def test_getters(self):
        # Create DggsPolygon object
        orig_cells = [DggsCell("0000"), DggsCell("1111"), DggsCell("2222")]
        linestring = DggsLinestring()
        linestring.add_cell(orig_cells[0])
        polygon = DggsPolygon(linestring)
        linestring = DggsLinestring()
        linestring.add_cell(orig_cells[1])
        polygon.add_inner_ring(linestring)
        linestring = DggsLinestring()
        linestring.add_cell(orig_cells[2])
        polygon.add_inner_ring(linestring)
        # Test DggsPolygon object
        outer_ring_cells = polygon.get_outer_ring().get_cells()
        self.assertEqual(outer_ring_cells[0], orig_cells[0])
        inner_rings = polygon.get_inner_rings()
        self.assertEqual(len(inner_rings), 2)
        inner_ring_cells = inner_rings[0].get_cells()
        self.assertEqual(inner_ring_cells[0], orig_cells[1])
        inner_ring_cells = inner_rings[1].get_cells()
        self.assertEqual(inner_ring_cells[0], orig_cells[2])

    def test_errors(self):
        # Error with outer ring
        with self.assertRaises(ValueError):
            DggsPolygon("Not a DggsLinestring")
        # Error with inner ring
        polygon = DggsPolygon(DggsLinestring())
        with self.assertRaises(ValueError):
            polygon.add_inner_ring("Not a DggsLinestring")



# -*- coding: utf-8 -*-

"""
This Python 3.3 module implements the cubic Snyder equal area map projection (CSEA) as described in [Snyd1992]_.
.. [Snyd1992] J. P. Snyder, An equal-area map projection for polyhedral globes, Cartographica 29 (1992), 10--21.
CHANGELOG:
- Alexander Raichev (AR), 2013-01-31: Initial version.
- AR, 2013-02-08: Fixed a bug in the xy-positions of the square faces.
- AR, 2013-07-23: Ported to Python 3.3.
NOTE:
All lengths are measured in meters and all angles are measured in radians
unless indicated otherwise.
By 'ellipsoid' below, i mean an oblate ellipsoid of revolution.
"""
# *****************************************************************************
#       Copyright (C) 2012 Alexander Raichev <alex.raichev@gmail.com>
#
#  Distributed under the terms of the GNU Lesser General Public License (LGPL)
#                  http://www.gnu.org/licenses/
# *****************************************************************************

# Import third-party modules.
from numpy import (
    pi,
    rad2deg,
    deg2rad,
    floor,
    sin,
    cos,
    arccos,
    arctan,
    arctan2,
    sqrt,
    array,
)

# Import my modules.
from .utils import auth_lat, auth_rad

g = arctan(sqrt(2))  # Snyder approximates this as 54.73561032 degrees.
tang = sqrt(2)
G = pi / 3
theta = pi / 4  # = 2*pi divided by twice the number of sides of a square
cottheta = 1
alpha = pi / 2  # = 2*(pi/2 - theta)
R_prime = sqrt(pi / 6)  # = (1/tang)*sqrt(2*(G - theta)/(sin(theta)*cos(theta)))
A = pi / 2 - g
face_centers_lonlat = [
    (-2 * pi / 3, A),
    (-pi / 3, -A),
    (0, A),
    (pi / 3, -A),
    (2 * pi / 3, A),
    (pi, -A),
]
B = sqrt(pi / 3)  # = R_prime*tang
face_centers_xy = [
    (-2 * B, B / 2),
    (-B, -B / 2),
    (0, B / 2),
    (B, -B / 2),
    (2 * B, B / 2),
    (3 * B, -B / 2),
]
EPS = 1e-12  # Fuzz to avoid rounding errors.


def csea_sphere(lam, phi):
    r"""
    Compute the signature function of the icosohedral Snyder equal area
    (ISEA) map projection of the unit sphere.

    INPUT:

    - `lam, phi` - Geodetic longitude-latitude coordinates in radians.
      Assume -pi <= `lam` < pi and -pi/2 <= `phi` <= pi/2.
    """
    # (lam, phi) lies in one of these two squares:
    face_1 = int(floor((lam + pi) / (pi / 3)))
    face_2 = (face_1) - 1 % 6
    for i in [face_1, face_2]:
        lam_0, phi_0 = face_centers_lonlat[i]
        z = arccos(sin(phi_0) * sin(phi) + cos(phi_0) * cos(phi) * (cos(lam - lam_0)))
        if z > g:
            # (lam, phi) does not lie in this square
            continue
        # Might have found the correct square face.
        Az = arctan2(
            cos(phi) * sin(lam - lam_0),
            cos(phi_0) * sin(phi) - sin(phi_0) * cos(phi) * cos(lam - lam_0),
        )
        j = 0
        while Az <= 0 or Az > alpha:
            if Az <= 0:
                Az += alpha
                j += 1
            else:
                Az -= alpha
                j -= 1
        q = arctan(tang / (cos(Az) + sin(Az) * cottheta))
        if z > q + EPS:
            # (lam, phi) does not lie in this square
            continue
        # Found the correct square face.
        H = arccos(sin(Az) * sin(G) * cos(g) - cos(Az) * cos(G))
        A_G = Az + G + H - pi
        Az_prime = arctan2(2 * A_G, (R_prime * tang) ** 2 - 2 * A_G * cottheta)
        d_prime = R_prime * tang / (cos(Az_prime) + sin(Az_prime) * cottheta)
        f = d_prime / (2 * R_prime * sin(q / 2))
        rho = 2 * R_prime * f * sin(z / 2)
        Az_prime -= j * alpha
        x = rho * sin(Az_prime) + face_centers_xy[i][0]
        y = rho * cos(Az_prime) + face_centers_xy[i][1]
        return x, y


def csea_sphere_inverse(x, y):
    r"""
    Compute the inverse of csea_sphere().
    INPUT:

    - `x, y` - Planar coordinates in meters in the image of the
      ISEA projection of the unit sphere.

    NOTES:

    Not implemented here nor in PROJ.4.
    """
    pass


def csea_ellipsoid(lam, phi, e=0):
    r"""
    Compute the signature functions of the ISEA may projection of an oblate
    ellipsoid with eccentricity `e` whose authalic sphere is the unit sphere.

    INPUT:

    - `lam, phi` - Geodetic longitude-latitude coordinates in radians.
      Assume -pi <= `lam` < pi and -pi/2 <= `phi` <= pi/2.
    - `e` - Eccentricity of the oblate ellipsoid.
    """
    beta = auth_lat(phi, e, radians=True)
    return csea_sphere(lam, beta)


def csea_ellipsoid_inverse(x, y, e=0):
    r"""
    Compute the inverse of csea_ellipsoid().

    INPUT:

    - `x, y` - Planar coordinates in meters in the image of the
      ISEA projection of the ellipsoid with eccentricity `e` whose
      authalic sphere is the unit sphere.

    NOTES:

    Not implemented here nor in PROJ.4.
    """
    pass


def csea(a=1, e=0):
    r"""
    Return a function object that wraps the ISEA projection and its inverse
    of an ellipsoid with major radius `a` and eccentricity `e`.

    OUTPUT:

    - A function object of the form f(u, v, radians=False, inverse=False).
    """
    R_A = auth_rad(a, e)

    def f(u, v, radians=False, inverse=False):
        if not inverse:
            lam, phi = u, v
            if not radians:
                # Convert to radians.
                lam, phi = deg2rad([lam, phi])
            return tuple(R_A * array(csea_ellipsoid(lam, phi, e=e)))
        else:
            # Scale down to R_A = 1.
            x, y = array((u, v)) / R_A
            lam, phi = array(csea_ellipsoid_inverse(x, y, e=e))
            if not radians:
                # Convert to degrees.
                lam, phi = rad2deg([lam, phi])
            return lam, phi

    return f



import argparse
import platform

if platform.system() == "Linux":
    from vgrid.utils.dggrid4py import DGGRIDv7, dggs_types


def dggrid_stats(dggrid_instance, dggs_type, resolution, output):
    dggrid_metrics = dggrid_instance.grid_stats_table(dggs_type, resolution)
    if output:
        dggrid_metrics.to_csv(output, index=False)
    else:
        print(dggrid_metrics)


def main():
    if platform.system() == "Linux":
        parser = argparse.ArgumentParser(description="Export or display DGGRID stats.")
        parser.add_argument(
            "-t",
            "--dggs_type",
            choices=dggs_types,
            help="Select a DGGS type from the available options.",
        )
        parser.add_argument(
            "-r", "--resolution", type=int, required=True, help="resolution"
        )
        parser.add_argument("-o", "--output", help="Output CSV file name.")
        args = parser.parse_args()

        dggrid_instance = DGGRIDv7(
            executable="/usr/local/bin/dggrid",
            working_dir=".",
            capture_logs=False,
            silent=True,
            tmp_geo_out_legacy=False,
            debug=False,
        )
        dggs_type = args.dggs_type
        resolution = args.resolution
        output = args.output
        try:
            dggrid_stats(dggrid_instance, dggs_type, resolution, output)
        except Exception:
            print(
                "Please ensure that the -r <resolution> are set appropriately, and there is an excutable DGGRID located at /usr/local/bin/dggrid. Please install DGGRID following instructions from https://github.com/sahrk/DGGRID/blob/master/INSTALL.md"
            )
    else:
        print(
            "dggrid only works on Linux with an excutable DGGRID at /usr/local/bin/dggrid. Please install DGGRID following instructions from https://github.com/sahrk/DGGRID/blob/master/INSTALL.md"
        )


if __name__ == "__main__":
    main()
